@@ -134,28 +134,15 @@ const char *strnstr(const char *haystack, const char *needle, int len) {
 
 #ifndef HAVE_STRCASESTR
 const char *strcasestr(const char *haystack, const char *needle) {
-	int len;
-
-	if ((haystack == NULL) || (needle == NULL)) {
-		return NULL;
-	}
-
-	len = strlen(needle);
-
-	return strncasestr(haystack, needle, len);
-}
-#endif
-
-#ifndef HAVE_STRNCASESTR
-const char *strncasestr(const char *haystack, const char *needle, int len) {
-	int i, steps, needle_len;
+	int i, steps;
+	size_t needle_len;
 
 	if ((haystack == NULL) || (needle == NULL)) {
 		return NULL;
 	}
 
 	needle_len = strlen(needle);
-	steps = len - needle_len + 1;
+	steps = strlen(haystack) - needle_len + 1;
 
 	for (i = 0; i < steps; i++) {
 		if (strncasecmp(haystack + i, needle, needle_len) == 0) {
