@@ -26,4 +26,12 @@
 
 #define TIMER_OFF   0
 
+#define POLL_EVENT_BITS (POLLIN | POLLPRI | POLLHUP)
+
+#if defined(__GNUC__)
+#define UNUSED(x) x __attribute__((unused))
+#else
+#define UNUSED(x) x
+#endif
+
 #endif
